@@ -48,6 +48,7 @@ FLAGS=flags.FLAGS
 # test_module_foo, and module_bar instead of test_module_bar.
 import test_module_foo as module_foo
 import test_module_bar as module_bar
+import test_module_baz as module_baz
 
 def MultiLineEqual(expected_help, help):
   """Returns True if expected_help == help.  Otherwise returns False
@@ -507,7 +508,7 @@ class FlagsUnitTest(unittest.TestCase):
       "--testspacelist [] --x 10 "
       "--noexec --quack "
       "--test1 "
-      "--testget1 --no? --nodebug --nohelp --nohelpshort --nohelpxml "
+      "--testget1 --tmod_baz_x --no? --nodebug --nohelp --nohelpshort --nohelpxml "
       "--noq --notest0 --notestget2 "
       "--notestget3 --notestnone")
 
@@ -533,7 +534,7 @@ class FlagsUnitTest(unittest.TestCase):
       "--testspacelist [] --x 10 "
       "--debug --noexec --quack "
       "--test1 "
-      "--testget1 --no? --nohelp --nohelpshort --nohelpxml "
+      "--testget1 --tmod_baz_x --no? --nohelp --nohelpshort --nohelpxml "
       "--noq --notest0 --notestget2 "
       "--notestget3 --notestnone")
 
@@ -598,15 +599,15 @@ class FlagsUnitTest(unittest.TestCase):
 
     # Make sure allow_override doesn't work with None default
     try:
-      flags.DEFINE_boolean("dup3", 0, "runhelp d31", short_name='u',
+      flags.DEFINE_boolean("dup3", 0, "runhelp d31", short_name='u3',
                            allow_override=0)
       flag = FLAGS.FlagDict()['dup3']
       self.assertEqual(flag.default, 0)
 
-      flags.DEFINE_boolean("dup3", None, "runhelp d32", short_name='u',
+      flags.DEFINE_boolean("dup3", None, "runhelp d32", short_name='u3',
                            allow_override=1)
       raise AssertionError('Cannot override a flag with a default of None')
-    except flags.DuplicateFlag, e:
+    except flags.DuplicateFlagCannotPropagateNoneToSwig, e:
       pass
 
     # Make sure that when we override, the help string gets updated correctly
@@ -628,6 +629,12 @@ class FlagsUnitTest(unittest.TestCase):
     self.assertEqual("new1" in FLAGS.FlagDict(), True)
     self.assertEqual("new2" in FLAGS.FlagDict(), True)
 
+    # Then test that removing those flags works
+    FLAGS.RemoveFlagValues(new_flags)
+    self.assertEqual(len(FLAGS.FlagDict()), old_len)
+    self.assertFalse("new1" in FLAGS.FlagDict())
+    self.assertFalse("new2" in FLAGS.FlagDict())
+
     # Make sure AppendFlagValues works with flags with shortnames.
     new_flags = flags.FlagValues()
     flags.DEFINE_boolean("new3", 0, "runhelp n3", flag_values=new_flags)
@@ -642,6 +649,13 @@ class FlagsUnitTest(unittest.TestCase):
     self.assertTrue("n4" in FLAGS.FlagDict())
     self.assertEqual(FLAGS.FlagDict()['n4'], FLAGS.FlagDict()['new4'])
 
+    # Then test removing them
+    FLAGS.RemoveFlagValues(new_flags)
+    self.assertEqual(len(FLAGS.FlagDict()), old_len)
+    self.assertFalse("new3" in FLAGS.FlagDict())
+    self.assertFalse("new4" in FLAGS.FlagDict())
+    self.assertFalse("n4" in FLAGS.FlagDict())
+
     # Make sure AppendFlagValues fails on duplicates
     flags.DEFINE_boolean("dup4", 0, "runhelp d41")
     new_flags = flags.FlagValues()
@@ -774,60 +788,69 @@ class FlagsUnitTest(unittest.TestCase):
     FLAGS.__delattr__('UnitTestNumber')
     FLAGS.__delattr__('UnitTestList')
 
+  def _ReadFlagsFromFiles(self, argv, force_gnu):
+    return argv[:1] + FLAGS.ReadFlagsFromFiles(argv[1:], force_gnu=force_gnu)
+
   #### Flagfile Unit Tests ####
   def testMethod_flagfiles_1(self):
     """ Test trivial case with no flagfile based options. """
     self.__DeclareSomeFlags()
-    fake_cmd_line = 'fooScript --UnitTestBoolFlag'
-    fake_argv = fake_cmd_line.split(' ')
-    FLAGS(fake_argv)
-    self.assertEqual( FLAGS.UnitTestBoolFlag, 1)
-    self.assertEqual( fake_argv, FLAGS.ReadFlagsFromFiles(fake_argv))
-    self._UndeclareSomeFlags()
+    try:
+      fake_cmd_line = 'fooScript --UnitTestBoolFlag'
+      fake_argv = fake_cmd_line.split(' ')
+      FLAGS(fake_argv)
+      self.assertEqual( FLAGS.UnitTestBoolFlag, 1)
+      self.assertEqual( fake_argv, self._ReadFlagsFromFiles(fake_argv, False))
+    finally:
+      self._UndeclareSomeFlags()
   # end testMethodOne
 
   def testMethod_flagfiles_2(self):
     """Tests parsing one file + arguments off simulated argv"""
     self.__DeclareSomeFlags()
-    tmp_files = self._SetupTestFiles()
-    # specify our temp file on the fake cmd line
-    fake_cmd_line = 'fooScript --q --flagfile=%s' % tmp_files[0]
-    fake_argv = fake_cmd_line.split(' ')
+    try:
+      tmp_files = self._SetupTestFiles()
+      # specify our temp file on the fake cmd line
+      fake_cmd_line = 'fooScript --q --flagfile=%s' % tmp_files[0]
+      fake_argv = fake_cmd_line.split(' ')
 
-    # We should see the original cmd line with the file's contents spliced in.
-    # Note that these will be in REVERSE order from order encountered in file
-    # This is done so arguements we encounter sooner will have priority.
-    expected_results = ['fooScript',
-                          '--UnitTestMessage1=tempFile1!',
-                          '--UnitTestNumber=54321',
-                          '--noUnitTestBoolFlag',
-                          '--q']
-    test_results = FLAGS.ReadFlagsFromFiles(fake_argv)
-    self.assertEqual(expected_results, test_results)
-    self._RemoveTestFiles(tmp_files)
-    self._UndeclareSomeFlags()
+      # We should see the original cmd line with the file's contents spliced in.
+      # Note that these will be in REVERSE order from order encountered in file
+      # This is done so arguements we encounter sooner will have priority.
+      expected_results = ['fooScript',
+                            '--UnitTestMessage1=tempFile1!',
+                            '--UnitTestNumber=54321',
+                            '--noUnitTestBoolFlag',
+                            '--q']
+      test_results = self._ReadFlagsFromFiles(fake_argv, False)
+      self.assertEqual(expected_results, test_results)
+    finally:
+      self._RemoveTestFiles(tmp_files)
+      self._UndeclareSomeFlags()
   # end testTwo def
 
   def testMethod_flagfiles_3(self):
     """Tests parsing nested files + arguments of simulated argv"""
     self.__DeclareSomeFlags()
-    tmp_files = self._SetupTestFiles()
-    # specify our temp file on the fake cmd line
-    fake_cmd_line = ('fooScript --UnitTestNumber=77 --flagfile=%s'
-                     % tmp_files[1])
-    fake_argv = fake_cmd_line.split(' ')
+    try:
+      tmp_files = self._SetupTestFiles()
+      # specify our temp file on the fake cmd line
+      fake_cmd_line = ('fooScript --UnitTestNumber=77 --flagfile=%s'
+                       % tmp_files[1])
+      fake_argv = fake_cmd_line.split(' ')
 
-    expected_results = ['fooScript',
-                          '--UnitTestMessage1=tempFile1!',
-                          '--UnitTestNumber=54321',
-                          '--noUnitTestBoolFlag',
-                          '--UnitTestMessage2=setFromTempFile2',
-                          '--UnitTestNumber=6789a',
-                          '--UnitTestNumber=77']
-    test_results = FLAGS.ReadFlagsFromFiles(fake_argv)
-    self.assertEqual(expected_results, test_results)
-    self._RemoveTestFiles(tmp_files)
-    self._UndeclareSomeFlags()
+      expected_results = ['fooScript',
+                            '--UnitTestMessage1=tempFile1!',
+                            '--UnitTestNumber=54321',
+                            '--noUnitTestBoolFlag',
+                            '--UnitTestMessage2=setFromTempFile2',
+                            '--UnitTestNumber=6789a',
+                            '--UnitTestNumber=77']
+      test_results = self._ReadFlagsFromFiles(fake_argv, False)
+      self.assertEqual(expected_results, test_results)
+    finally:
+      self._RemoveTestFiles(tmp_files)
+      self._UndeclareSomeFlags()
   # end testThree def
 
   def testMethod_flagfiles_4(self):
@@ -835,20 +858,106 @@ class FlagsUnitTest(unittest.TestCase):
       This test should print a warning to stderr of some sort.
     """
     self.__DeclareSomeFlags()
-    tmp_files = self._SetupTestFiles()
-    # specify our temp file on the fake cmd line
-    fake_cmd_line = ('fooScript --flagfile=%s --noUnitTestBoolFlag'
-                     % tmp_files[2])
-    fake_argv = fake_cmd_line.split(' ')
-    expected_results = ['fooScript',
-                          '--UnitTestMessage1=setFromTempFile3',
-                          '--UnitTestBoolFlag',
-                          '--noUnitTestBoolFlag' ]
+    try:
+      tmp_files = self._SetupTestFiles()
+      # specify our temp file on the fake cmd line
+      fake_cmd_line = ('fooScript --flagfile=%s --noUnitTestBoolFlag'
+                       % tmp_files[2])
+      fake_argv = fake_cmd_line.split(' ')
+      expected_results = ['fooScript',
+                            '--UnitTestMessage1=setFromTempFile3',
+                            '--UnitTestBoolFlag',
+                            '--noUnitTestBoolFlag' ]
 
-    test_results = FLAGS.ReadFlagsFromFiles(fake_argv)
-    self.assertEqual(expected_results, test_results)
-    self._RemoveTestFiles(tmp_files)
-    self._UndeclareSomeFlags()
+      test_results = self._ReadFlagsFromFiles(fake_argv, False)
+      self.assertEqual(expected_results, test_results)
+    finally:
+      self._RemoveTestFiles(tmp_files)
+      self._UndeclareSomeFlags()
+
+  def testMethod_flagfiles_5(self):
+    """Test that --flagfile parsing respects the '--' end-of-options marker."""
+    self.__DeclareSomeFlags()
+    try:
+      tmp_files = self._SetupTestFiles()
+      # specify our temp file on the fake cmd line
+      fake_cmd_line = 'fooScript --SomeFlag -- --flagfile=%s' % tmp_files[0]
+      fake_argv = fake_cmd_line.split(' ')
+      expected_results = ['fooScript',
+                          '--SomeFlag',
+                          '--',
+                          '--flagfile=%s' % tmp_files[0]]
+
+      test_results = self._ReadFlagsFromFiles(fake_argv, False)
+      self.assertEqual(expected_results, test_results)
+    finally:
+      self._RemoveTestFiles(tmp_files)
+      self._UndeclareSomeFlags()
+
+  def testMethod_flagfiles_6(self):
+    """Test that --flagfile parsing stops at non-options (non-GNU behavior)."""
+    self.__DeclareSomeFlags()
+    try:
+      tmp_files = self._SetupTestFiles()
+      # specify our temp file on the fake cmd line
+      fake_cmd_line = ('fooScript --SomeFlag some_arg --flagfile=%s'
+                       % tmp_files[0])
+      fake_argv = fake_cmd_line.split(' ')
+      expected_results = ['fooScript',
+                          '--SomeFlag',
+                          'some_arg',
+                          '--flagfile=%s' % tmp_files[0]]
+
+      test_results = self._ReadFlagsFromFiles(fake_argv, False)
+      self.assertEqual(expected_results, test_results)
+    finally:
+      self._RemoveTestFiles(tmp_files)
+      self._UndeclareSomeFlags()
+
+  def testMethod_flagfiles_7(self):
+    """Test that --flagfile parsing skips over a non-option (GNU behavior)."""
+    self.__DeclareSomeFlags()
+    try:
+      FLAGS.UseGnuGetOpt()
+      tmp_files = self._SetupTestFiles()
+      # specify our temp file on the fake cmd line
+      fake_cmd_line = ('fooScript --SomeFlag some_arg --flagfile=%s'
+                       % tmp_files[0])
+      fake_argv = fake_cmd_line.split(' ')
+      expected_results = ['fooScript',
+                          '--UnitTestMessage1=tempFile1!',
+                          '--UnitTestNumber=54321',
+                          '--noUnitTestBoolFlag',
+                          '--SomeFlag',
+                          'some_arg']
+
+      test_results = self._ReadFlagsFromFiles(fake_argv, False)
+      self.assertEqual(expected_results, test_results)
+    finally:
+      self._RemoveTestFiles(tmp_files)
+      self._UndeclareSomeFlags()
+
+  def testMethod_flagfiles_8(self):
+    """Test that --flagfile parsing respects force_gnu=True."""
+    self.__DeclareSomeFlags()
+    try:
+      tmp_files = self._SetupTestFiles()
+      # specify our temp file on the fake cmd line
+      fake_cmd_line = ('fooScript --SomeFlag some_arg --flagfile=%s'
+                       % tmp_files[0])
+      fake_argv = fake_cmd_line.split(' ')
+      expected_results = ['fooScript',
+                          '--UnitTestMessage1=tempFile1!',
+                          '--UnitTestNumber=54321',
+                          '--noUnitTestBoolFlag',
+                          '--SomeFlag',
+                          'some_arg']
+
+      test_results = self._ReadFlagsFromFiles(fake_argv, True)
+      self.assertEqual(expected_results, test_results)
+    finally:
+      self._RemoveTestFiles(tmp_files)
+      self._UndeclareSomeFlags()
 
   def test_flagfiles_user_path_expansion(self):
     """Test that user directory referenced paths (ie. ~/foo) are correctly
@@ -1179,17 +1288,19 @@ class FlagsUnitTest(unittest.TestCase):
     assert len(argv) == 1, "wrong number of arguments pulled"
     assert argv[0]=='./program', "program name not preserved"
 
-  def test_main_module_help(self):
-    """Test MainModuleHelp()"""
-    help = FLAGS.MainModuleHelp()
+  def test_module_help(self):
+    """Test ModuleHelp()."""
+    helpstr = FLAGS.ModuleHelp(module_baz)
 
-    # When this test is invoked on behalf of flags_unittest_2_2,
-    # the main module has not defined any flags. Since there's
-    # no easy way to run this script in our test environment
-    # directly from python2.2, don't bother to test the output
-    # of MainModuleHelp() in that scenario.
-    if sys.version.startswith('2.2.'):
-      return
+    expected_help = "\n" + module_baz.__name__ + ":" + """
+  --[no]tmod_baz_x: Boolean flag.
+    (default: 'true')"""
+
+    self.assertMultiLineEqual(expected_help, helpstr)
+
+  def test_main_module_help(self):
+    """Test MainModuleHelp()."""
+    helpstr = FLAGS.MainModuleHelp()
 
     expected_help = "\n" + sys.argv[0] + ':' + """
   --[no]debug: debughelp
@@ -1252,7 +1363,7 @@ class FlagsUnitTest(unittest.TestCase):
   -z,--[no]zoom1: runhelp z1
     (default: 'false')"""
 
-    if not MultiLineEqual(expected_help, help):
+    if not MultiLineEqual(expected_help, helpstr):
       self.fail()
 
   def test_create_flag_errors(self):
@@ -1339,6 +1450,15 @@ class FlagsUnitTest(unittest.TestCase):
     """
     return [f.name for f in flag_values._GetKeyFlagsForModule(module)]
 
+  def _AssertListsHaveSameElements(self, list_1, list_2):
+    # Checks that two lists have the same elements with the same
+    # multiplicity, in possibly different order.
+    list_1 = list(list_1)
+    list_1.sort()
+    list_2 = list(list_2)
+    list_2.sort()
+    self.assertListEqual(list_1, list_2)
+
   def testKeyFlags(self):
     # Before starting any testing, make sure no flags are already
     # defined for module_foo and module_bar.
@@ -1354,11 +1474,13 @@ class FlagsUnitTest(unittest.TestCase):
       # Part 1. Check that all flags defined by module_foo are key for
       # that module, and similarly for module_bar.
       for module in [module_foo, module_bar]:
-        self.assertListEqual(FLAGS._GetFlagsDefinedByModule(module),
-                             FLAGS._GetKeyFlagsForModule(module))
+        self._AssertListsHaveSameElements(
+            FLAGS._GetFlagsDefinedByModule(module),
+            FLAGS._GetKeyFlagsForModule(module))
         # Also check that each module defined the expected flags.
-        self.assertListEqual(self._GetNamesOfDefinedFlags(module),
-                             module.NamesOfDefinedFlags())
+        self._AssertListsHaveSameElements(
+            self._GetNamesOfDefinedFlags(module),
+            module.NamesOfDefinedFlags())
 
       # Part 2. Check that flags.DECLARE_key_flag works fine.
       # Declare that some flags from module_bar are key for
@@ -1366,12 +1488,14 @@ class FlagsUnitTest(unittest.TestCase):
       module_foo.DeclareKeyFlags()
 
       # Check that module_foo has the expected list of defined flags.
-      self.assertListEqual(self._GetNamesOfDefinedFlags(module_foo),
-                           module_foo.NamesOfDefinedFlags())
+      self._AssertListsHaveSameElements(
+          self._GetNamesOfDefinedFlags(module_foo),
+          module_foo.NamesOfDefinedFlags())
 
       # Check that module_foo has the expected list of key flags.
-      self.assertListEqual(self._GetNamesOfKeyFlags(module_foo),
-                           module_foo.NamesOfDeclaredKeyFlags())
+      self._AssertListsHaveSameElements(
+          self._GetNamesOfKeyFlags(module_foo),
+          module_foo.NamesOfDeclaredKeyFlags())
 
       # Part 3. Check that flags.ADOPT_module_key_flags works fine.
       # Trigger a call to flags.ADOPT_module_key_flags(module_bar)
@@ -1380,9 +1504,10 @@ class FlagsUnitTest(unittest.TestCase):
       module_foo.DeclareExtraKeyFlags()
 
       # Check that module_foo has the expected list of key flags.
-      self.assertListEqual(self._GetNamesOfKeyFlags(module_foo),
-                           module_foo.NamesOfDeclaredKeyFlags() +
-                           module_foo.NamesOfDeclaredExtraKeyFlags())
+      self._AssertListsHaveSameElements(
+          self._GetNamesOfKeyFlags(module_foo),
+          module_foo.NamesOfDeclaredKeyFlags() +
+          module_foo.NamesOfDeclaredExtraKeyFlags())
     finally:
       module_foo.RemoveFlags()
 
@@ -1411,9 +1536,10 @@ class FlagsUnitTest(unittest.TestCase):
 
     # Check that all flags defined by module_bar are key for that
     # module, and that module_bar defined the expected flags.
-    self.assertListEqual(fv._GetFlagsDefinedByModule(module_bar),
-                         fv._GetKeyFlagsForModule(module_bar))
-    self.assertListEqual(
+    self._AssertListsHaveSameElements(
+        fv._GetFlagsDefinedByModule(module_bar),
+        fv._GetKeyFlagsForModule(module_bar))
+    self._AssertListsHaveSameElements(
         self._GetNamesOfDefinedFlags(module_bar, flag_values=fv),
         module_bar.NamesOfDefinedFlags())
 
@@ -1428,21 +1554,31 @@ class FlagsUnitTest(unittest.TestCase):
     flag_name_2 = names_of_flags_defined_by_bar[2]
 
     flags.DECLARE_key_flag(flag_name_0, flag_values=fv)
-    self.assertListEqual(
+    self._AssertListsHaveSameElements(
         self._GetNamesOfKeyFlags(main_module, flag_values=fv),
         [flag_name_0])
 
     flags.DECLARE_key_flag(flag_name_2, flag_values=fv)
-    self.assertListEqual(
+    self._AssertListsHaveSameElements(
         self._GetNamesOfKeyFlags(main_module, flag_values=fv),
         [flag_name_0, flag_name_2])
 
+    # Try with a special (not user-defined) flag too:
+    flags.DECLARE_key_flag('undefok', flag_values=fv)
+    self._AssertListsHaveSameElements(
+        self._GetNamesOfKeyFlags(main_module, flag_values=fv),
+        [flag_name_0, flag_name_2, 'undefok'])
+
     flags.ADOPT_module_key_flags(module_bar, flag_values=fv)
-    key_flags = self._GetNamesOfKeyFlags(main_module, flag_values=fv)
-    # Order is irrelevant; hence, we sort both lists before comparison.
-    key_flags.sort()
-    names_of_flags_defined_by_bar.sort()
-    self.assertListEqual(key_flags, names_of_flags_defined_by_bar)
+    self._AssertListsHaveSameElements(
+        self._GetNamesOfKeyFlags(main_module, flag_values=fv),
+        names_of_flags_defined_by_bar + ['undefok'])
+
+    # Adopt key flags from the module google3.pyglib.flags itself.
+    flags.ADOPT_module_key_flags(flags, flag_values=fv)
+    self._AssertListsHaveSameElements(
+        self._GetNamesOfKeyFlags(main_module, flag_values=fv),
+        names_of_flags_defined_by_bar + ['flagfile', 'undefok'])
 
   def testMainModuleHelpWithKeyFlags(self):
     # Similar to test_main_module_help, but this time we make sure to
@@ -1564,6 +1700,110 @@ class FlagsUnitTest(unittest.TestCase):
         global_dict['module_name'],
         'test_module_bar')
 
+  def test_GetCallingModuleWithIteritemsError(self):
+    # This test checks that _GetCallingModule is using
+    # sys.modules.items(), instead of .iteritems().
+    orig_sys_modules = sys.modules
+
+    # Mock sys.modules: simulates error produced by importing a module
+    # in paralel with our iteration over sys.modules.iteritems().
+    class SysModulesMock(dict):
+      def __init__(self, original_content):
+        dict.__init__(self, original_content)
+
+      def iteritems(self):
+        # Any dictionary method is fine, but not .iteritems().
+        raise RuntimeError('dictionary changed size during iteration')
+
+    sys.modules = SysModulesMock(orig_sys_modules)
+    try:
+      # _GetCallingModule should still work as expected:
+      self.assertEqual(flags._GetCallingModule(), sys.argv[0])
+      self.assertEqual(
+          module_foo.GetModuleName(),
+          'test_module_foo')
+    finally:
+      sys.modules = orig_sys_modules
+
+
+class FlagsErrorMessagesTest(unittest.TestCase):
+  """Testing special cases for integer and float flags error messages."""
+
+  def setUp(self):
+    # make sure we are using the old, stupid way of parsing flags.
+    FLAGS.UseGnuGetOpt(False)
+    self.flag_values = flags.FlagValues()
+
+  def testIntegerErrorText(self):
+    # Make sure we get proper error text
+    flags.DEFINE_integer('positive', 4, 'non-negative flag', lower_bound=1,
+                         flag_values=self.flag_values)
+    flags.DEFINE_integer('non_negative', 4, 'positive flag', lower_bound=0,
+                         flag_values=self.flag_values)
+    flags.DEFINE_integer('negative', -4, 'negative flag', upper_bound=-1,
+                         flag_values=self.flag_values)
+    flags.DEFINE_integer('non_positive', -4, 'non-positive flag', upper_bound=0,
+                         flag_values=self.flag_values)
+    flags.DEFINE_integer('greater', 19, 'greater-than flag', lower_bound=4,
+                         flag_values=self.flag_values)
+    flags.DEFINE_integer('smaller', -19, 'smaller-than flag', upper_bound=4,
+                         flag_values=self.flag_values)
+    flags.DEFINE_integer('usual', 4, 'usual flag', lower_bound=0,
+                         upper_bound=10000, flag_values=self.flag_values)
+    flags.DEFINE_integer('another_usual', 0, 'usual flag', lower_bound=-1,
+                         upper_bound=1, flag_values=self.flag_values)
+
+    self._CheckErrorMessage('positive', '-4', 'a positive integer')
+    self._CheckErrorMessage('non_negative', '-4', 'a non-negative integer')
+    self._CheckErrorMessage('negative', '0', 'a negative integer')
+    self._CheckErrorMessage('non_positive', '4', 'a non-positive integer')
+    self._CheckErrorMessage('usual', '-4', 'an integer in the range [0, 10000]')
+    self._CheckErrorMessage('another_usual', '4',
+                            'an integer in the range [-1, 1]')
+    self._CheckErrorMessage('greater', '-5', 'integer >= 4')
+    self._CheckErrorMessage('smaller', '5', 'integer <= 4')
+
+  def testFloatErrorText(self):
+    # Make sure we get proper error text
+    flags.DEFINE_float('positive', 4, 'non-negative flag', lower_bound=1,
+                       flag_values=self.flag_values)
+    flags.DEFINE_float('non_negative', 4, 'positive flag', lower_bound=0,
+                       flag_values=self.flag_values)
+    flags.DEFINE_float('negative', -4, 'negative flag', upper_bound=-1,
+                       flag_values=self.flag_values)
+    flags.DEFINE_float('non_positive', -4, 'non-positive flag', upper_bound=0,
+                       flag_values=self.flag_values)
+    flags.DEFINE_float('greater', 19, 'greater-than flag', lower_bound=4,
+                       flag_values=self.flag_values)
+    flags.DEFINE_float('smaller', -19, 'smaller-than flag', upper_bound=4,
+                       flag_values=self.flag_values)
+    flags.DEFINE_float('usual', 4, 'usual flag', lower_bound=0,
+                       upper_bound=10000, flag_values=self.flag_values)
+    flags.DEFINE_float('another_usual', 0, 'usual flag', lower_bound=-1,
+                       upper_bound=1, flag_values=self.flag_values)
+
+    self._CheckErrorMessage('positive', '0.5', 'number >= 1')
+    self._CheckErrorMessage('non_negative', '-4.0', 'a non-negative number')
+    self._CheckErrorMessage('negative', '0.5', 'number <= -1')
+    self._CheckErrorMessage('non_positive', '4.0', 'a non-positive number')
+    self._CheckErrorMessage('usual', '-4.0', 'a number in the range [0, 10000]')
+    self._CheckErrorMessage('another_usual', '4.0',
+                            'a number in the range [-1, 1]')
+    self._CheckErrorMessage('greater', '-5.0', 'number >= 4')
+    self._CheckErrorMessage('smaller', '5.0', 'number <= 4')
+
+  def _CheckErrorMessage(self, flag_name, flag_value, expected_message_suffix):
+    try:
+      second_arg = '--%s=%s' % (flag_name, flag_value)
+      argv = ('./program', second_arg)
+      self.flag_values(argv)
+      raise AssertionError('Bounds exception not raised!')
+    except flags.IllegalFlagValue, e:
+      expected = ('flag --%(name)s=%(value)s: %(value)s is not %(suffix)s' %
+                  {'name': flag_name, 'value': flag_value,
+                   'suffix': expected_message_suffix})
+      self.assertEquals(str(e), expected)
+
 
 def main():
   unittest.main()
